import json
from PyQt5.QtCore import QVariant
from qgis.PyQt.QtCore import Qt
from qgis.PyQt.QtWidgets import QDialog
from qgis.core import (
    QgsApplication,
    QgsCoordinateReferenceSystem,
    QgsCoordinateTransform,
    QgsFeature,
    QgsField,
    QgsGeometry,
    QgsLineString,
    QgsPointXY,
    QgsPolygon,
    QgsProject,
    QgsVectorLayer,
    QgsWkbTypes,
)
from qgis.gui import QgsRubberBand, QgsMapTool
from qgis.utils import iface
from ..tools.networking import NetworkRequestTask


class BagExtractPanel:
    def __init__(self, dialog: QDialog, plugin) -> None:
        self._plugin = plugin
        self._dialog = dialog

        # self.oldtool = None
        self.startPoint = self.endPoint = None
        self.isDownloading = False

    def setup_panel(self) -> None:
        print("Setup BAG extract panel")

        self.display_error()

        self._dialog.bag_extract_click_button.clicked.connect(
            self.clicked_btn_rectangle_tool
        )
        self._dialog.bag_extract_download_button.clicked.connect(
            self.clicked_btn_download
        )

    def make_active_panel(self):
        print("isochrone make active")
        self.display_error()

    def display_error(self):
        if self._plugin.error_message is None:
            self._dialog.bag_extract_error_label.setVisible(False)
        else:
            self._dialog.bag_extract_error_label.setText(self._plugin.error_message)
            self._dialog.bag_extract_error_label.setVisible(True)

    def clicked_btn_rectangle_tool(self):
        canvas = iface.mapCanvas()
        # self.oldtool = canvas.mapTool()
        my_rectangle_tool = MyRectangleTool(
            canvas, callback_rectangle_created=self.callback_rectangle_created
        )
        canvas.setMapTool(my_rectangle_tool)

    def callback_rectangle_created(self, my_rectangle_tool: "MyRectangleTool"):
        sourceCrs = QgsProject.instance().crs()
        destCrs = QgsCoordinateReferenceSystem.fromEpsgId(4326)
        tr = QgsCoordinateTransform(sourceCrs, destCrs, QgsProject.instance())

        self.startPoint = tr.transform(my_rectangle_tool.startPoint)
        self.endPoint = tr.transform(my_rectangle_tool.endPoint)

        self._dialog.bag_extract_start_x.setText(f"{self.startPoint.x():.6f}")
        self._dialog.bag_extract_start_y.setText(f"{self.startPoint.y():.6f}")
        self._dialog.bag_extract_end_x.setText(f"{self.endPoint.x():.6f}")
        self._dialog.bag_extract_end_y.setText(f"{self.endPoint.y():.6f}")

    def clicked_btn_download(self):
        if self.isDownloading:
            print("wait for download to complete then press again")
            return

        if self.startPoint is None or self.endPoint is None:
            print("Select an area first")
            return

        print("clicked_btn_download")

        point1 = QgsPointXY(self.startPoint.x(), self.startPoint.y())
        point2 = QgsPointXY(self.startPoint.x(), self.endPoint.y())
        point3 = QgsPointXY(self.endPoint.x(), self.endPoint.y())
        point4 = QgsPointXY(self.endPoint.x(), self.startPoint.y())

        points = [point1, point2, point3, point4, point1]

        polygon = QgsPolygon(QgsLineString(points))

        url = self._plugin.api_url + "/routing/v1/bag_extract.geojson"
        request_body = {"geometry": json.loads(polygon.asJson())}
        request_body["geometry"]["crs"] = {
            "type": "name",
            "properties": {"name": "urn:ogc:def:crs:EPSG::4326"},
        }
        headers = {
            "Content-type": "application/json",
            "Authorization": self._plugin.api_key,
        }

        self.isDownloading = True
        self._dialog.bag_extract_download_button.setText("Downloading... Please wait")

        globals()["stratopo_bag_extract_download_task"] = NetworkRequestTask(
            url,
            request_headers=headers,
            request_body=request_body,
            callback=self.callback_download_task_finished,
        )
        QgsApplication.taskManager().addTask(
            globals()["stratopo_bag_extract_download_task"]
        )

    def callback_download_task_finished(self, response):
        data = response.json()

        pointLayer = QgsVectorLayer("Point?crs=epsg:28992", "BAG", "memory")
        pointLayerDataProvider = pointLayer.dataProvider()

        pointLayerDataProvider.addAttributes(
            [
                QgsField("openbareruimtenaam", QVariant.String),
                QgsField("verkorteopenbareruimtenaam", QVariant.String),
                QgsField("openbareruimte_id", QVariant.String),
                QgsField("huisnummer", QVariant.String),
                QgsField("huisletter", QVariant.String),
                QgsField("huisnummertoevoeging", QVariant.String),
                QgsField("postcode", QVariant.String),
                QgsField("nummeraanduiding_id", QVariant.String),
                QgsField("woonplaatsnaam", QVariant.String),
                QgsField("woonplaatscode", QVariant.String),
                QgsField("gemeentenaam", QVariant.String),
                QgsField("gemeentecode", QVariant.String),
                QgsField("provincienaam", QVariant.String),
                QgsField("provinciecode", QVariant.String),
                QgsField("nevenadres", QVariant.String),
                QgsField("typeadresseerbaarobject", QVariant.String),
                QgsField("adresseerbaarobject_status", QVariant.String),
                QgsField("opp_adresseerbaarobject_m2", QVariant.String),
                QgsField("aantal_nad_per_adresobject", QVariant.String),
                QgsField("adresseerbaarobject_id", QVariant.String),
                QgsField("geopunt", QVariant.String),
                QgsField("x", QVariant.String),
                QgsField("y", QVariant.String),
                QgsField("lon", QVariant.String),
                QgsField("lat", QVariant.String),
                QgsField("woonfunctie", QVariant.String),
                QgsField("bijeenkomstfunctie", QVariant.String),
                QgsField("celfunctie", QVariant.String),
                QgsField("gezondheidszorgfunctie", QVariant.String),
                QgsField("industriefunctie", QVariant.String),
                QgsField("kantoorfunctie", QVariant.String),
                QgsField("logiesfunctie", QVariant.String),
                QgsField("onderwijsfunctie", QVariant.String),
                QgsField("sportfunctie", QVariant.String),
                QgsField("winkelfunctie", QVariant.String),
                QgsField("overige_gebruiksfunctie", QVariant.String),
                QgsField("aantal_pand_relaties_dit_vbo", QVariant.String),
                QgsField("pand_id", QVariant.String),
                QgsField("aantal_vbo_relaties_dit_pnd", QVariant.String),
                QgsField("vbo_pnd_1_op_1", QVariant.String),
                QgsField("opp_pand", QVariant.String),
                QgsField("woningtype", QVariant.String),
                QgsField("omtrek_pand", QVariant.String),
                QgsField("verhouding_opp_vbo_opp_pnd", QVariant.String),
                QgsField("pandstatus", QVariant.String),
                QgsField("bouwjaar", QVariant.String),
                QgsField("uniq_key", QVariant.String),
                QgsField("pchnhlht", QVariant.String),
                QgsField("pchnhlht_uniek", QVariant.String),
                QgsField("pchn_uniek", QVariant.String),
                QgsField("rangorde_pchn", QVariant.String),
            ]
        )

        pointLayer.updateFields()

        for feature in data["features"]:
            f = QgsFeature(pointLayer.fields())
            pt = QgsPointXY(*feature["geometry"]["coordinates"])
            f.setGeometry(QgsGeometry.fromPointXY(pt))
            for key, value in feature["properties"].items():
                f.setAttribute(key, value)

            pointLayerDataProvider.addFeature(f)

        pointLayer.updateExtents()

        QgsProject.instance().addMapLayer(pointLayer)

        self.isDownloading = False
        self._dialog.bag_extract_download_button.setText("Download BAG objects within rectangle")


class MyRectangleTool(QgsMapTool):
    def __init__(self, canvas, callback_rectangle_created):
        super().__init__(canvas)
        self.rubberBand = QgsRubberBand(canvas, True)
        self.rubberBand.setWidth(3)
        self.rubberBand.setColor(Qt.blue)
        self.rubberBand.setFillColor(Qt.transparent)
        self.startPoint = self.endPoint = None
        self.isEmittingPoint = False
        self.callback_rectangle_created = callback_rectangle_created

    def canvasPressEvent(self, e):
        self.startPoint = self.endPoint = self.toMapCoordinates(e.pos())
        self.isEmittingPoint = True
        self.showRect(self.startPoint, self.endPoint)

    def canvasReleaseEvent(self, e):
        self.isEmittingPoint = False
        self.callback_rectangle_created(self)

    def canvasMoveEvent(self, e):
        if self.rubberBand is not None:

            if not self.isEmittingPoint:
                return

            self.endPoint = self.toMapCoordinates(e.pos())
            self.showRect(self.startPoint, self.endPoint)

    def showRect(self, startPoint, endPoint):
        self.rubberBand.reset(QgsWkbTypes.PolygonGeometry)

        if startPoint.x() == endPoint.x() or startPoint.y() == endPoint.y():
            return

        point1 = QgsPointXY(startPoint.x(), startPoint.y())
        point2 = QgsPointXY(startPoint.x(), endPoint.y())
        point3 = QgsPointXY(endPoint.x(), endPoint.y())
        point4 = QgsPointXY(endPoint.x(), startPoint.y())
        self.rubberBand.addPoint(point1, False)
        self.rubberBand.addPoint(point2, False)
        self.rubberBand.addPoint(point3, False)
        self.rubberBand.addPoint(point4, True)  # true to update canvas
        self.rubberBand.show()

    def deactivate(self) -> None:
        super().deactivate()
        self.rubberBand.reset()
        del self.rubberBand
        self.rubberBand = None
