from qgis.PyQt.QtWidgets import QDialog
from qgis.PyQt.QtCore import QVariant
from qgis.core import QgsVectorLayer, QgsFeature, QgsPointXY, QgsGeometry, QgsProject, QgsField

from .panels import Panels
import requests
import re

class BodPanel():
    def __init__(self, dialog: QDialog, plugin) -> None:
        self._plugin = plugin
        self._dialog = dialog

        self.selectedLat = ""
        self.selectedLon = ""

    def setup_panel(self) -> None:
        print("Setup Bod panel")
        self.display_error()
        self._dialog.bod_calculate_push_button.clicked.connect(self.clicked_btn)
        self._dialog.bod_get_coordinate_push_button.clicked.connect(self.get_coordinate_from_selected_point)

    def make_active_panel(self):
        self.display_error()

    def display_error(self):
        if self._plugin.error_message is None:
            self._dialog.bod_error_label.setVisible(False)
        else:
            self._dialog.bod_error_label.setText(self._plugin.error_message)
            self._dialog.bod_error_label.setVisible(True)

    def get_coordinate_from_selected_point(self):
        for layer in QgsProject.instance().mapLayers().values():
            if type(layer) is QgsVectorLayer:
                selected_features = layer.selectedFeatures()
                for feature in selected_features:
                    try:
                        geometry = QgsGeometry.asPoint(feature.geometry())#get the geometry of the feature
                        point = QgsPointXY(geometry)
                        # TODO: Save selected point and add BOD data to it later 
                        self.selectedLon = point.x() # lan
                        self.selectedLat = point.y() # lat
                        self._dialog.bod_lat_line_edit.setText(str(self.selectedLat))
                        self._dialog.bod_lon_line_edit.setText(str(self.selectedLon))
                        return   
                    except:
                        pass
    

    def clicked_btn(self):

        bod_lat_line_edit = self._dialog.bod_lat_line_edit.text()
        bod_lon_line_edit = self._dialog.bod_lon_line_edit.text()
        
        bod_type_radio_button_intercitytrain_checked = self._dialog.bod_type_radio_button_intercitytrain.isChecked()
        bod_type_radio_button_regionaltrain_checked = self._dialog.bod_type_radio_button_regionaltrain.isChecked()
        bod_type_radio_button_bus_checked = self._dialog.bod_type_radio_button_bus.isChecked()
        bod_type_radio_button_tram_checked = self._dialog.bod_type_radio_button_tram.isChecked()
        bod_type_radio_button_metro_checked = self._dialog.bod_type_radio_button_metro.isChecked()
        bod_type_radio_button_citycenter_checked = self._dialog.bod_type_radio_button_citycenter.isChecked()
        bod_type_radio_button_shoppingcenter_checked = self._dialog.bod_type_radio_button_shoppingcenter.isChecked()
        bod_type_radio_button_smallshop_checked = self._dialog.bod_type_radio_button_smallshop.isChecked()
        bod_type_radio_button_elementaryschool_checked = self._dialog.bod_type_radio_button_elementaryschool.isChecked()
        bod_type_radio_button_highschool_checked = self._dialog.bod_type_radio_button_highschool.isChecked()
        bod_type_radio_button_university_checked = self._dialog.bod_type_radio_button_university.isChecked()
        bod_type_radio_button_commuting_checked = self._dialog.bod_type_radio_button_commuting.isChecked()

        bod_travel_time_line_edit = self._dialog.bod_travel_time_line_edit.text()

        # Get Lat Long
        if not bool(re.match("^[0-9]{1,3}(?:[\.,][0-9]+)?$", bod_lat_line_edit)):
            print("Wrong lat input")
            self._plugin.error_message = 'Error in latitude'
            # self.showBodWindow()
            self.display_error()
            return
        if not bool(re.match("^[0-9]{1,3}(?:[\.,][0-9]+)?$", bod_lon_line_edit)):
            print("Wrong lon input")
            self._plugin.error_message = 'Error in longitude'
            # self.showBodWindow()
            self.display_error()
            return

        lat = float(bod_lat_line_edit.replace(',', '.')) 
        lon = float(bod_lon_line_edit.replace(',', '.'))

        # bod_type
        bod_type = ""
        if bod_type_radio_button_intercitytrain_checked:
            bod_type = "intercitytrain"
        if bod_type_radio_button_regionaltrain_checked:
            bod_type = "regionaltrain"
        if bod_type_radio_button_bus_checked:
            bod_type = "bus"
        if bod_type_radio_button_tram_checked:
            bod_type = "tram"
        if bod_type_radio_button_metro_checked:
            bod_type = "metro"
        if bod_type_radio_button_citycenter_checked:
            bod_type = "citycenter"
        if bod_type_radio_button_shoppingcenter_checked:
            bod_type = "shoppingcenter"
        if bod_type_radio_button_smallshop_checked:
            bod_type = "smallshop"
        if bod_type_radio_button_elementaryschool_checked:
            bod_type = "elementaryschool"
        if bod_type_radio_button_highschool_checked:
            bod_type = "hogeschool"
        if bod_type_radio_button_university_checked:
            bod_type = "university"
        if bod_type_radio_button_commuting_checked:
            bod_type = "commuting"
        
        if bod_type == "":
            print("One form of modality should be checked")
            self._plugin.error_message = 'Error: modality not selected'
            # self.showBodWindow()
            self.display_error()
            return

        # travelTime 
        if not bool(re.match("^[0-9]+$", bod_travel_time_line_edit)):
            print("Wrong travel time input")
            self._plugin.error_message = 'Travel time should be a number'
            # self.showBodWindow()
            self.display_error()
            return

        minutes = int(bod_travel_time_line_edit)
        if minutes < 1 or minutes > 60:
            print("Travel time should be between 1 and 60")
            self._plugin.error_message = 'Travel time should be between 1 and 60'
            # self.showBodWindow()
            self.display_error()
            return
        
        # Make request
        url = self._plugin.api_url+"/routing/v1/bod"
        requestBody = {
            "latitude": lat,
            "longitude": lon,
            "bod_type": bod_type,
            "minutes": minutes
        }
        headers = {'Content-type': 'application/json', 'Authorization': self._plugin.api_key}
        r = requests.post(url, json=requestBody, headers=headers)

        if r.status_code == 200:
            self.drawBod(r.json())
            self._plugin.error_message = None
            self._dialog.hide()
            self.display_error()
        elif r.status_code == 403 or r.status_code == 401:
            print("Error form server: ", r.status_code)
            self._plugin.error_message = 'Provide a valid API key and try again'
            # self.showConfigurationWindow()
            self._dialog.switch_panel(Panels.Settings)
        else:
            print("Error form server: ", r.status_code)
            self._plugin.error_message = 'Server error'
            # self.showBodWindow()
            self.display_error()


    def drawBod(self, geoJson):

        if "aantal_bag_objecten" in geoJson and \
            "bod_score" in geoJson and \
            "bod_type" in geoJson and \
            "latitude" in geoJson and \
            "longitude" in geoJson and \
            "minutes" in geoJson and \
            "nod_score" in geoJson and \
            "sum_gem_hh_gr" in geoJson:

            # point
            pointLayer =  QgsVectorLayer('Point', 'BOD_Point' , "memory")
            pointDataProvider = pointLayer.dataProvider() 
            pointDataProvider.addAttributes([QgsField("aantal_bag_objecten", QVariant.Int), QgsField("bod_score", QVariant.Double), QgsField("bod_type", QVariant.String), QgsField("minutes", QVariant.Int), QgsField("nod_score", QVariant.Double), QgsField("sum_gem_hh_gr", QVariant.Double)])
            pointLayer.updateFields()

            feature = QgsFeature()
            point1 = QgsPointXY(geoJson["longitude"], geoJson["latitude"])
            feature.setGeometry(QgsGeometry.fromPointXY(point1))

            attributes = [0, None, "", 0, None, None]
            attributes[0] = geoJson["aantal_bag_objecten"]
            attributes[1] = geoJson["bod_score"]
            attributes[2] = geoJson["bod_type"]
            attributes[3] = geoJson["minutes"]
            attributes[4] = geoJson["nod_score"]
            attributes[5] = geoJson["sum_gem_hh_gr"]
             
            feature.setAttributes(attributes)
            pointDataProvider.addFeatures([feature])
            pointLayer.updateExtents()

            # add layers
            QgsProject.instance().addMapLayers([pointLayer])